@@ -1,5 +1,5 @@
 from __future__ import print_function
-import unittest,os,tempfile,sys,glob,subprocess
+import unittest,os,tempfile,sys,glob,subprocess,multiprocessing
 from stat import *
 from testcdo import *
 import numpy as np
@@ -12,7 +12,7 @@ if 'CDF_MOD' in os.environ:
 else:
   CDF_MOD = 'netcdf4'
 
-HOSTNAME = 'nearly'
+HOSTNAME = 'luthien'
 DATA_DIR = os.environ.get('HOME')+'/local/data'
 
 SHOW = 'SHOW' in os.environ
@@ -46,8 +46,8 @@ class CdoTest(unittest.TestCase):
 
     def testVersions(self):
         cdo = Cdo()
-        self.assertEqual('1.2.6',cdo.__version__())
-        self.assertEqual('1.6.9',cdo.version())
+        self.assertEqual('1.3.0',cdo.__version__())
+        self.assertTrue(parse_version('1.7.0') <= parse_version(cdo.version()))
 
     def testCDO(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -69,7 +69,9 @@ class CdoTest(unittest.TestCase):
         self.assertTrue("sinfov" in cdo.operators)
         self.assertTrue("for" in cdo.operators)
         self.assertTrue("mask" in cdo.operators)
-        self.assertTrue("studentt" in cdo.operators)
+        if (parse_version('1.7.0') >= parse_version(cdo.version())):
+            self.assertTrue("studentt" in cdo.operators)
+        self.assertTrue(len(cdo.operators) > 700)
 
     def test_getOperators(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -436,8 +438,7 @@ class CdoTest(unittest.TestCase):
 
 
     def test_fillmiss(self):
-      cdo = Cdo(cdfMod='netcdf4')
-      if HOSTNAME == os.popen('hostname').read().strip():
+        cdo = Cdo(cdfMod='netcdf4')
         if 'CDO' in os.environ:
           cdo.setCdo(os.environ.get('CDO'))
 
@@ -460,13 +461,20 @@ class CdoTest(unittest.TestCase):
         arOrg = cdo.copy(input = rand,returnMaArray = 'v')
         arWmr = cdo.setrtomiss(missRange,input = rand,output = withMissRange,returnMaArray='v')
         arFm  = cdo.fillmiss(            input = withMissRange,returnMaArray = 'v')
-        arFm1s= cdo.fillmiss2(2,        input = withMissRange,returnMaArray = 'v',output='foo.nc')
+        arFm1s= cdo.fillmiss2(2,         input = withMissRange,returnMaArray = 'v',output='foo.nc')
+        if 'setmisstonn' in cdo.operators:
+          arM2NN= cdo.setmisstonn(         input = withMissRange,returnMaArray = 'v',output='foo.nc')
 
-        plot(arOrg,title='org'        )#ofile='fmOrg.svg')
-        plot(arWmr,title='missing'    )#ofile='fmWmr.svg')
-        plot(arFm,title='fillmiss'    )#ofile='fmFm.svg')
-        plot(arFm1s,title='fillmiss1s')#ofile='fmFm2.svg')
-        #        os.system("convert +append %s %s %s %s fm_all.png "%('fm_org.png','fm_wmr.png','fm_fm.png','fm_fm1s.png
+        pool = multiprocessing.Pool(8)
+        pool.apply_async(plot, (arOrg, ),{"title":'org'      })#ofile='fmOrg')
+        pool.apply_async(plot, (arWmr, ),{"title":'missing'  })#ofile='fmWmr')
+        pool.apply_async(plot, (arFm,  ),{"title":'fillmiss' })#ofile= 'fmFm')
+        pool.apply_async(plot, (arFm1s,),{"title":'fillmiss2'})#ofile='fmFm2')
+        if 'setmisstonn' in cdo.operators:
+          pool.apply_async(plot, (arM2NN,), {"title":'setmisstonn'})#, ofile='fmsetMNN')
+
+        pool.close()
+        pool.join()
 
     def test_keep_coordinates(self):
         cdo = Cdo(cdfMod=CDF_MOD)
@@ -484,7 +492,47 @@ class CdoTest(unittest.TestCase):
 
           varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
           varOut = varOut.variables[ivar]
-          self.assertEqual(expected,varOut.coordinates)
+          expected = expected.split(' ')
+          expected.reverse()
+          self.assertEqual(expected,varOut.coordinates.split(' '))
+
+    def testTmp(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        import glob
+        tempDir = tempfile.gettempdir()
+        tempfilesStart = glob.glob('{0}/cdoPy*'.format(tempDir))
+        tempfilesStart.sort()
+        tempfilesEnd   = glob.glob('{0}/cdoPy**'.format(tempDir))
+        tempfilesEnd.sort()
+        self.assertEqual(tempfilesStart,tempfilesEnd)
+
+        self.test_combine()
+        tempfilesEnd = glob.glob('{0}/cdoPy**'.format(tempDir))
+        tempfilesEnd.sort()
+        self.assertEqual(tempfilesStart,tempfilesEnd)
+    def test_readArray(self):
+        cdo = Cdo(cdfMod=CDF_MOD)
+        ifile = cdo.enlarge('r44x35',
+                            input=' -stdatm,0,100,1000',
+                            options='-f nc')
+        self.assertEqual((3,35,44), cdo.readArray(ifile, 'T').shape)
+
+    def test_log(self):
+        cmd = '-fldmean -mul -random,r20x20 -topo,r20x20'
+        print('# logging with a real file')
+        cdo = Cdo(cdfMod=CDF_MOD,logging = True,logFile='foo.log')
+        cdo.topo()
+        cdo.temp()
+        cdo.sinfov(input=cmd)
+        cdo.showLog()
+
+        print('# logging with in-memory stringio')
+        cdo = Cdo(cdfMod=CDF_MOD,logging = True)
+        cdo.topo()
+        cdo.temp()
+        cdo.sinfov(input=cmd)
+        cdo.showLog()
+
 
     if HOSTNAME == os.popen('hostname').read().strip():
         def test_icon_coords(self):
@@ -496,12 +544,12 @@ class CdoTest(unittest.TestCase):
             if ('scipy' == cdo.cdfMod ):
               expected = b'clon clat'
             else:
-              expected =  'clon clat'
+              expected =  u'clon clat'
             self.assertEqual(expected,varIn.coordinates)
 
-
             varOut =cdo.readCdf(cdo.selname(ivar,input=ifile))
             varOut = varOut.variables[ivar]
+            expected =  u'clat clon'
             self.assertEqual(expected,varOut.coordinates)
         def testCall(self):
             cdo = Cdo(cdfMod=CDF_MOD)
@@ -514,25 +562,6 @@ class CdoTest(unittest.TestCase):
 
             self.assertEqual(sorted(['lat','lon','for','time']),sorted(list(cdf.variables.keys())))
 
-        def testTmp(self):
-            cdo = Cdo(cdfMod=CDF_MOD)
-            import glob
-            tempfilesStart = glob.glob('/tmp/cdoPy*')
-            tempfilesStart.sort()
-            tempfilesEnd   = glob.glob('/tmp/cdoPy**')
-            tempfilesEnd.sort()
-            self.assertEqual(tempfilesStart,tempfilesEnd)
-
-            self.test_combine()
-            tempfilesEnd = glob.glob('/tmp/cdoPy**')
-            tempfilesEnd.sort()
-            self.assertEqual(tempfilesStart,tempfilesEnd)
-        def test_readArray(self):
-            cdo = Cdo(cdfMod=CDF_MOD)
-            ifile = cdo.enlarge('r44x35',
-                                input=' -stdatm,0,100,1000',
-                                options='-f nc')
-            self.assertEqual((3,35,44), cdo.readArray(ifile, 'T').shape)
 
         def test_phc(self):
            ifile = DATA_DIR+'/icon/phc.nc'
@@ -542,11 +571,22 @@ class CdoTest(unittest.TestCase):
            #          output=ifile,
            #          options='-O')
            s = cdo.sellonlatbox(0,30,0,90, input="-chname,SO,s,TempO,t " + ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
-           plot(s[0,:,:],ofile='org',title='org')
+           plot(np.flipud(s[0,:,:]),ofile='org',title='orgiginal')
            sfmo = cdo.sellonlatbox(0,30,0,90, input="-fillmiss -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-           plot(sfmo[0,:,:],ofile='fm',title='fm')
+           plot(np.flipud(sfmo[0,:,:]),ofile='fm',title='fillmiss')
            sfm = cdo.sellonlatbox(0,30,0,90, input="-fillmiss2 -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
-           plot(sfm[0,:,:],ofile='fm2',title='fm2')
+           plot(np.flipud(sfm[0,:,:]),ofile='fm2',title='fillmiss2')
+           ssetmisstonn = cdo.sellonlatbox(0,30,0,90, input="-setmisstonn -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+           plot(np.flipud(ssetmisstonn[0,:,:]),ofile='setmisstonn',title='setmisstonn')
+           #global plot
+          #s_global = cdo.chname('SO,s,TempO,t',input=ifile,output='my_phc.nc',returnMaArray='s',options='-f nc')
+          #plot(s_global[0,:,:],ofile='org_global',title='org_global')
+          #sfmo_global = cdo.fillmiss(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          #plot(sfmo_global[0,:,:],ofile='fm_global',title='fm_global')
+          #sfm_global = cdo.fillmiss2(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          #plot(sfm_global[0,:,:],ofile='fm2_global',title='fm2_global')
+          #ssetmisstonn_global = cdo.setmisstonn(input=" -chname,SO,s,TempO,t " + ifile,returnMaArray='s',options='-f nc')
+          #plot(ssetmisstonn_global[0,:,:],ofile='setmisstonn_global',title='setmisstonn_global')
 
             
 
